import * as tslib_1 from "tslib";
import { FormGroup, FormControl, FormBuilder } from '@angular/forms';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { MetadataStorage, Validator, getFromContainer, ValidationTypes, validateSync } from 'class-validator';
import { plainToClass } from 'class-transformer';
import 'reflect-metadata';
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var DynamicFormGroup = /** @class */ (function (_super) {
    tslib_1.__extends(DynamicFormGroup, _super);
    /**
     * @param {?} factoryModel
     * @param {?} fields
     */
    function DynamicFormGroup(factoryModel, fields) {
        var _this = _super.call(this, {}) || this;
        _this.factoryModel = factoryModel;
        _this.fields = fields;
        _this.customValidateErrors = new BehaviorSubject({});
        _this.fields = _this.clearFields(_this.fields);
        return _this;
    }
    /**
     * @template TModel
     * @param {?} factoryModel
     * @param {?} fields
     * @return {?}
     */
    DynamicFormGroup.getClassValidators = function (factoryModel, fields) {
        var /** @type {?} */ validationMetadatas = getFromContainer(MetadataStorage).getTargetValidationMetadatas(factoryModel, '');
        var /** @type {?} */ formGroupFields = {};
        var /** @type {?} */ validator = new Validator();
        Object.keys(fields).forEach(function (key) {
            var /** @type {?} */ formGroupField = formGroupFields[key];
            if (formGroupField === undefined) {
                formGroupField = Array.isArray(fields[key]) ? fields[key] : [];
            }
            if (!Array.isArray(fields[key])) {
                if (fields[key] instanceof DynamicFormGroup) {
                    fields[key].object = fields[key].fields;
                    formGroupField.push(fields[key]);
                }
                else {
                    formGroupField.push(fields[key]);
                }
            }
            validationMetadatas.forEach(function (validationMetadata) {
                if (validationMetadata.propertyName === key) {
                    switch (validationMetadata.type) {
                        /*
                                                case ValidationTypes.IS_NOT_EMPTY: {
                                                    formGroupField.push(Validators.required);
                                                    break;
                                                }
                                                case ValidationTypes.IS_EMAIL: {
                                                    formGroupField.push(Validators.required);
                                                    break;
                                                }*/
                        default: {
                            for (var /** @type {?} */ typeKey in ValidationTypes) {
                                if (ValidationTypes.hasOwnProperty(typeKey)) {
                                    if (validationMetadata.type === ValidationTypes[typeKey] &&
                                        validator[validationMetadata.type] === undefined &&
                                        validationMetadata.type === 'nestedValidation' &&
                                        typeKey === 'NESTED_VALIDATION') {
                                        var /** @type {?} */ nestedValidate = function (c) {
                                            var /** @type {?} */ isValid = (c.parent && c.parent.value) ?
                                                validateSync(c.value, { validationError: { target: false } }).length === 0 : true;
                                            return isValid ? null : {
                                                nestedValidate: {
                                                    valid: false,
                                                    type: validationMetadata.type
                                                }
                                            };
                                        };
                                        formGroupField.push(nestedValidate);
                                    }
                                    if (validationMetadata.type === ValidationTypes[typeKey] &&
                                        validator[validationMetadata.type] === undefined &&
                                        validationMetadata.type === 'customValidation' &&
                                        typeKey === 'CUSTOM_VALIDATION') {
                                        var /** @type {?} */ customValidation = function (c) {
                                            // todo: refactor
                                            var /** @type {?} */ object = (c.parent instanceof DynamicFormGroup) ?
                                                ((c.parent)).object :
                                                (c.parent ? c.parent.value : {});
                                            if (object) {
                                                object[key] = c.value;
                                            }
                                            var /** @type {?} */ validateErrors = (c.parent && c.parent.value) ?
                                                validateSync(object, { validationError: { target: false } }) : [];
                                            var /** @type {?} */ isValid = validateErrors.filter(function (error) {
                                                if (error.children.length &&
                                                    error.children.filter(function (children) { return children.property === key; })) {
                                                    return true;
                                                }
                                                return true;
                                            }).length === 0;
                                            return isValid ? null : {
                                                customValidation: {
                                                    valid: false,
                                                    type: validationMetadata.type
                                                }
                                            };
                                        };
                                        formGroupField.push(customValidation);
                                    }
                                    if (validationMetadata.type === ValidationTypes[typeKey] &&
                                        validator[validationMetadata.type] !== undefined) {
                                        var /** @type {?} */ customValidate = function (c) {
                                            if (!c) {
                                                return null;
                                            }
                                            var /** @type {?} */ isValid = c.parent && c.parent.value ? validator[validationMetadata.type](c.value) : true;
                                            return isValid ? null : {
                                                customValidate: {
                                                    valid: false,
                                                    type: validationMetadata.type
                                                }
                                            };
                                        };
                                        formGroupField.push(customValidate);
                                    }
                                }
                            }
                            break;
                        }
                    }
                }
            });
            if (formGroupField[0] instanceof DynamicFormGroup) {
                formGroupFields[key] = formGroupField[0];
            }
            else {
                formGroupFields[key] = [
                    formGroupField[0],
                    formGroupField.filter(function (item, index) { return index !== 0; })
                ];
            }
        });
        return formGroupFields;
    };
    /**
     * @param {?} fields
     * @return {?}
     */
    DynamicFormGroup.prototype.clearFields = function (fields) {
        var _this = this;
        var /** @type {?} */ newFields = {};
        Object.keys(fields).forEach(function (key) {
            if (fields[key] instanceof DynamicFormGroup) {
                newFields[key] = _this.clearFields(((fields[key])).fields);
            }
            else {
                newFields[key] = fields[key];
            }
        });
        return newFields;
    };
    Object.defineProperty(DynamicFormGroup.prototype, "object", {
        /**
         * @return {?}
         */
        get: function () {
            var _this = this;
            var /** @type {?} */ object = this._object ? this._object : new this.factoryModel();
            if (object !== undefined) {
                Object.keys(this.controls).forEach(function (key) {
                    if (_this.controls[key] instanceof DynamicFormGroup) {
                        object[key] = ((_this.controls[key])).value;
                    }
                    else {
                        object[key] = _this.controls[key].value;
                    }
                });
            }
            return plainToClass(this.factoryModel, object);
        },
        /**
         * @param {?} object
         * @return {?}
         */
        set: function (object) {
            var _this = this;
            this._object = plainToClass(this.factoryModel, object);
            var /** @type {?} */ currentValues = {};
            Object.keys(this.controls).forEach(function (key) {
                if (_this.controls[key] instanceof DynamicFormGroup) {
                    ((_this.controls[key])).object =
                        _this._object[key] !== undefined ? _this._object[key] : ((_this.controls[key])).object;
                }
                currentValues[key] = _this._object[key] !== undefined ? _this._object[key] : _this.controls[key].value;
            });
            var /** @type {?} */ currentObject = plainToClass(this.factoryModel, currentValues);
            this.setValue(currentObject);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} errors
     * @return {?}
     */
    DynamicFormGroup.prototype.transformValidationErrors = function (errors) {
        var _this = this;
        var /** @type {?} */ customErrors = {};
        errors.forEach(function (error) {
            if (error.constraints !== undefined) {
                Object.keys(error.constraints).forEach(function (key) {
                    if (!customErrors[error.property]) {
                        customErrors[error.property] = [];
                    }
                    if (customErrors[error.property].indexOf(error.constraints[key]) === -1) {
                        customErrors[error.property].push(error.constraints[key]);
                    }
                });
            }
            if (error.children !== undefined && error.children.length) {
                customErrors[error.property] = _this.transformValidationErrors(error.children);
            }
        });
        return customErrors;
    };
    /**
     * @param {?=} otherErrors
     * @return {?}
     */
    DynamicFormGroup.prototype.validate = function (otherErrors) {
        if (otherErrors === undefined) {
            otherErrors = [];
        }
        var /** @type {?} */ errors = validateSync(this.object, { validationError: { target: false } });
        this.customValidateErrors.next(this.transformValidationErrors(tslib_1.__spread(errors, otherErrors)));
    };
    /**
     * @return {?}
     */
    DynamicFormGroup.prototype.validateAllFormFields = function () {
        var _this = this;
        Object.keys(this.controls).forEach(function (field) {
            var /** @type {?} */ control = _this.get(field);
            if (control instanceof FormControl) {
                control.markAsTouched({ onlySelf: true });
            }
            else if (control instanceof DynamicFormGroup) {
                control.validateAllFormFields();
            }
        });
    };
    return DynamicFormGroup;
}(FormGroup));
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var DynamicFormBuilder = /** @class */ (function (_super) {
    tslib_1.__extends(DynamicFormBuilder, _super);
    function DynamicFormBuilder() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * @template TModel
     * @param {?} factoryModel
     * @param {?} controlsConfig
     * @param {?=} extra
     * @return {?}
     */
    DynamicFormBuilder.prototype.group = function (factoryModel, controlsConfig, extra) {
        var /** @type {?} */ dynamicFormGroup = new DynamicFormGroup(factoryModel, controlsConfig);
        var /** @type {?} */ formGroup = _super.prototype.group.call(this, DynamicFormGroup.getClassValidators(factoryModel, controlsConfig), extra);
        Object.keys(formGroup.controls).forEach(function (key) {
            dynamicFormGroup.addControl(key, formGroup.controls[key]);
        });
        dynamicFormGroup.valueChanges.subscribe(function (data) {
            dynamicFormGroup.validate();
        });
        return dynamicFormGroup;
    };
    return DynamicFormBuilder;
}(FormBuilder));
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var NgxDynamicFormBuilderShareds = [DynamicFormBuilder, DynamicFormGroup];
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */
export { DynamicFormBuilder, DynamicFormGroup, NgxDynamicFormBuilderShareds };
//# sourceMappingURL=ngx-dynamic-form-builder.js.map
